@@ -1,3 +1,4 @@
 from .shared_memory import from_shared_memory, to_shared_memory, free_memory, remove_shared_memory_in_session, \
-    SingleSharedArray, free_memory_in_session, to_file, from_file, object_to_shared_memory, object_from_shared_memory
+    SingleSharedArray, free_memory_in_session
 from .shared_memory import get_shared_pool, close_shared_pool
+from .shared_memory_v2 import to_file, from_file, object_to_shared_memory, object_from_shared_memory
\ No newline at end of file
