@@ -380,8 +380,12 @@ def to_shared_memory(object, name=None, use_python_backend=False):
     return name
 
 
-def remove_shared_memory(name):
-    shared_memories = [s.name.decode("utf-8") for s in sa.list()]
+def remove_shared_memory(name, limit_to_session=False):
+    if limit_to_session:
+        shared_memories = SHARED_MEMORIES_IN_SESSION
+    else:
+        shared_memories = [s.name.decode("utf-8") for s in sa.list()]
+
 
     n_deleted = 0
     for m in shared_memories:
@@ -406,6 +410,9 @@ def remove_shared_memory(name):
 
         n_deleted += 1
 
+    if n_deleted == 0:
+        logging.debug("Did not find anything to delete fro %s" % name)
+
 
 def remove_shared_memory_in_session():
     for name in SHARED_MEMORIES_IN_SESSION:
@@ -428,11 +435,22 @@ def free_memory():
 def free_memory_in_session():
     remove_shared_memory_in_session()
 
-def _run_numpy_based_function_on_shared_memory_arguments(function, arguments, interval):
+def _run_numpy_based_function_on_shared_memory_arguments(function, input_arguments, interval):
     start_time = time.perf_counter()
     start, end = interval
     #logging.info("Running on interval %d-%d" % (start, end))
-    arguments = [from_shared_memory(SingleSharedArray, a).array if type(a) == str else a for a in arguments]
+    arguments = []
+    for a in input_arguments:
+        if isinstance(a, str):
+            try:
+                object = from_shared_memory(SingleSharedArray, a).array
+                arguments.append(object)
+            except FileNotFoundError:
+                logging.debug("Did not find shared memory %s. Assuming this is a real string and not a shared memory name" % a)
+                arguments.append(a)
+        else:
+            arguments.append(a)
+
     sliced_arguments = []
     for argument in arguments:
         if isinstance(argument, np.ndarray) and argument.shape[0] != 1:
@@ -446,7 +464,7 @@ def _run_numpy_based_function_on_shared_memory_arguments(function, arguments, in
     return shared_memory_name
 
 
-def run_numpy_based_function_in_parallel(function, n_threads, arguments):
+def run_numpy_based_function_in_parallel(function, n_threads, arguments, chunks=None):
     # split every argument that is not an int
     # put each in shared memory
     # run function on chunk on separete thread, put in result array
@@ -463,7 +481,9 @@ def run_numpy_based_function_in_parallel(function, n_threads, arguments):
         # don't split ndarrays with shape[0] == 1
         if isinstance(argument, np.ndarray) and argument.shape[0] != 1:
             argument_id = str(np.random.randint(0, 10e15))
+            t0 = time.perf_counter()
             to_shared_memory(SingleSharedArray(argument), argument_id)
+            logging.info("Time to write to shared memory: %.3f" % (time.perf_counter()-t0))
             new_arguments.append(argument_id)
             if array_length != 0 and array_length != argument.shape[0]:
                 logging.error("Found argument with different shape")
@@ -481,20 +501,25 @@ def run_numpy_based_function_in_parallel(function, n_threads, arguments):
     t = time.perf_counter()
     pool = get_shared_pool(n_threads)  # Pool(n_threads)
 
-    intervals = list([int(i) for i in np.linspace(0, array_length, n_threads + 1)])
-    intervals = [(from_pos, to_pos) for from_pos, to_pos in zip(intervals[0:-1], intervals[1:])]
-    #logging.info("Will run on intervals %s" % intervals)
+    if chunks is None:
+        intervals = list([int(i) for i in np.linspace(0, array_length, n_threads + 1)])
+        intervals = [(from_pos, to_pos) for from_pos, to_pos in zip(intervals[0:-1], intervals[1:])]
+    else:
+        logging.debug("Using predefined chunks")
+        intervals = chunks
+
+    logging.info("Will run on intervals %s" % intervals)
 
     results = []
     for result_name in pool.starmap(_run_numpy_based_function_on_shared_memory_arguments, zip(repeat(function), repeat(new_arguments), intervals)):
         result = from_shared_memory(SingleSharedArray, result_name).array
         results.append(result)
-        remove_shared_memory(result_name)
+        sa.delete(result_name + "__array")
 
     # Free memory for arrays used
     for argument in new_arguments:
         if isinstance(argument, str):
-            remove_shared_memory(argument)
+            remove_shared_memory(argument, limit_to_session=True)
 
     t = time.perf_counter()
     results = np.concatenate(results)
